
/**
 * CartaoBonus  uma classe que modela um carto de crdito com 
 * uma diferena de comportamento em relaao a CartaoCredito e que se traduz no facto
 * de as despesas feitas acumularem pontos para posterior troca.
 * 
 * @author anr
 * @version Abril 2003
 */
public class CartaoBonus extends CartaoCredito
{
    // variveis de instncia
    private double pontos;

    /**
     * Construtor parametrizado de CartaoBonus.
     * O nmero de pontos inicial de um carto de crdito com mximo de dbito superior a 5000 EUR 
     *  1000, caso contrrio  de 250.
     */
    public CartaoBonus(String titular, String numero, String mes, String ano, double maxdeb) {
    
        //invocar o construtor da classe CartaoCredito para inicializar as suas variveis 
        super(titular,numero,mes,ano,maxdeb);
        if (this.getMaxDeb() > 5000)
          this.pontos = 1000;
        else
          this.pontos = 250;
    }
   
   
   /**
    * Construtor de cpia.
    * 
    * Utiliza o construtor de cpia definido na superclasse para inicializar as variveis
    * de instncia que foram definidas em CartaoCredito.
    * 
    */
      
   
    public CartaoBonus(CartaoBonus cb) {
      super(cb);
      this.pontos = cb.getPontos();
    }
   
   /**
    * Mtodo que devolve o total de pontos acumulado
    * 
    * @return  double com os pontos acumulados por invocao do mtodo gastar
    */
   
   public double getPontos() {
     return this.pontos;
   }
   
   /**
    * Mtodo que permite gastar dinheiro. Dado um montante e uma descrio de movimento
    * esta informao  guardada e os pontos actualizados.
    * Por cada Euro gasto atribuem-se dois pontos.
    * 
    * Este mtodo substitui o mtodo gastar da classe CartaoCredito
    */
   
   public void gastar(double quantia, String descr) {
     double pontos = quantia * 2;
     
     //invocar o mtodo gastar da superclasse para efectuar o cdigo l existente
     super.gastar(quantia,descr);
     //actualizar os pontos
     this.pontos += pontos; 
     
   }
   
   /**
    * Mtodo que devolve uma String com a representao do CartaoBonus a quem este mtodo
    *  enviado
    * 
    * @return String com formatao dos valores do estado de um CartaoBonus
    */
   
   public String toString() {
     //invocao da String com os valores das variveis de instncia da superclasse.
     //Evita-se assim a escrita de cdigo rigorosamente igual...
     String saux = super.toString();
     
     return saux+ "Pontos: " + this.pontos;
   }
   
   
   /**
    * Mtodo que devolve uma cpia (uma nova referncia) do objecto ao qual foi enviado.
    * 
    * 
    * @return Um novo objecto CartaoBonus devolvido como Object
    */
   
   public Object clone() {
     return new CartaoBonus(this);
     
   }  
   
   public boolean equals(Object cb) {
       if (this == cb) 
         return true;
       if (cb!=null && cb.getClass() == this.getClass()) {
         CartaoBonus cartao = (CartaoBonus)cb;
         return (super.equals(cartao) && this.pontos == cartao.getPontos());
        }
        else 
          return false;
   }
   
   
}
